@@ -36,8 +36,9 @@ int preserve_gid = 0;
 int preserve_times = 0;
 int update_only = 0;
 int cvs_exclude = 0;
+int dry_run=0;
 
-static int server = 0;
+int am_server = 0;
 static int sender = 0;
 static int recurse = 0;
 
@@ -48,7 +49,7 @@ static void report(int f)
   
   if (!verbose) return;
 
-  if (server && sender) {
+  if (am_server && sender) {
     write_int(f,read_total());
     write_int(f,write_total());
     write_int(f,total_size);
@@ -92,6 +93,8 @@ static void server_options(char **args,int *argc)
     argstr[x++] = 'b';
   if (update_only)
     argstr[x++] = 'u';
+  if (dry_run)
+    argstr[x++] = 'n';
   if (preserve_links)
     argstr[x++] = 'l';
   if (preserve_uid)
@@ -274,36 +277,38 @@ void do_server_recv(int argc,char *argv[])
   }
 
   if ((pid=fork()) == 0) {
-    if (verbose > 2)
-      fprintf(stderr,"generator starting pid=%d count=%d\n",
-	      (int)getpid(),flist->count);
-
-    for (i = 0; i < flist->count; i++) {
-      if (S_ISDIR(flist->files[i].mode)) {
-	if (mkdir(flist->files[i].name,flist->files[i].mode) != 0 && 
-	    errno != EEXIST) {	 
-	  fprintf(stderr,"mkdir %s: %s\n",
-		  flist->files[i].name,strerror(errno));
-	  exit(1);
-	}
-	continue;
-      }
-      fname = flist->files[i].name;
-      if (flist->count == 1 &&
-	  argc > 0)
-	fname = argv[0];
-      recv_generator(fname,flist,i,STDOUT_FILENO);
-    }
-    write_int(STDOUT_FILENO,-1);
-    write_flush(STDOUT_FILENO);
+    recv_files(STDIN_FILENO,flist,fname);
     if (verbose > 1)
-      fprintf(stderr,"generator wrote %d\n",write_total());
+      fprintf(stderr,"receiver read %d\n",read_total());
     exit(0);
   }
 
-  recv_files(STDIN_FILENO,flist,fname);
+  if (verbose > 2)
+    fprintf(stderr,"generator starting pid=%d count=%d\n",
+	    (int)getpid(),flist->count);
+
+  for (i = 0; i < flist->count; i++) {
+    if (S_ISDIR(flist->files[i].mode)) {
+      if (dry_run) continue;
+      if (mkdir(flist->files[i].name,flist->files[i].mode) != 0 && 
+	  errno != EEXIST) {	 
+	fprintf(stderr,"mkdir %s: %s\n",
+		flist->files[i].name,strerror(errno));
+	exit(1);
+      }
+      continue;
+    }
+    fname = flist->files[i].name;
+    if (flist->count == 1 &&
+	argc > 0)
+      fname = argv[0];
+    recv_generator(fname,flist,i,STDOUT_FILENO);
+  }
+  write_int(STDOUT_FILENO,-1);
+  write_flush(STDOUT_FILENO);
   if (verbose > 1)
-    fprintf(stderr,"receiver read %d\n",read_total());
+    fprintf(stderr,"generator wrote %d\n",write_total());
+
   waitpid(pid, &status, 0);
   exit(status);
 }
@@ -328,6 +333,7 @@ static void usage(FILE *f)
   fprintf(f,"-g, --group              preserve group\n");
   fprintf(f,"-D, --devices            preserve devices (root only)\n");
   fprintf(f,"-t, --times              preserve times\n");  
+  fprintf(f,"-n, --dry-run            show what would have been transferred\n");
   fprintf(f,"-B, --block-size SIZE    checksum blocking size\n");  
   fprintf(f,"-e, --rsh COMMAND        specify rsh replacement\n");
   fprintf(f,"-C, --cvs-exclude        auto ignore files in the same way CVS does\n");
@@ -343,7 +349,7 @@ static void usage(FILE *f)
 
 enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,OPT_EXCLUDE_FROM};
 
-static char *short_options = "oblpguDCtcahvre:B:";
+static char *short_options = "oblpguDCtcahvrne:B:";
 
 static struct option long_options[] = {
   {"version",     0,     0,    OPT_VERSION},
@@ -352,6 +358,7 @@ static struct option long_options[] = {
   {"exclude",     1,     0,    OPT_EXCLUDE},
   {"exclude-from",1,     0,    OPT_EXCLUDE_FROM},
   {"help",        0,     0,    'h'},
+  {"dry-run",     0,     0,    'n'},
   {"cvs-exclude", 0,     0,    'C'},
   {"archive",     0,     0,    'a'},
   {"checksum",    0,     0,    'c'},
@@ -416,6 +423,10 @@ int main(int argc,char *argv[])
 	  make_backups=1;
 	  break;
 
+	case 'n':
+	  dry_run=1;
+	  break;
+
 	case 'C':
 	  cvs_exclude=1;
 	  break;
@@ -483,11 +494,11 @@ int main(int argc,char *argv[])
 	  break;
 
 	case OPT_SERVER:
-	  server = 1;
+	  am_server = 1;
 	  break;
 
 	case OPT_SENDER:
-	  if (!server) {
+	  if (!am_server) {
 	    usage(stderr);
 	    exit(1);
 	  }
@@ -516,8 +527,10 @@ int main(int argc,char *argv[])
       argv++;
     }
 
+    if (dry_run)
+      verbose = MAX(verbose,1);
 
-    if (server) {
+    if (am_server) {
       int version = read_int(STDIN_FILENO);
       if (version != PROTOCOL_VERSION) {
 	fprintf(stderr,"protocol version mismatch %d %d\n",
@@ -646,32 +659,37 @@ int main(int argc,char *argv[])
       }
     }
 
+
     if ((pid2=fork()) == 0) {
-      for (i = 0; i < flist->count; i++) {
-	if (S_ISDIR(flist->files[i].mode)) {
-	  if (mkdir(flist->files[i].name,flist->files[i].mode) != 0 &&
-	      errno != EEXIST) {
-	    fprintf(stderr,"mkdir %s : %s\n",
-		    flist->files[i].name,strerror(errno));
-	  }
-	  continue;
-	}
-	recv_generator(local_name?local_name:flist->files[i].name,
-		       flist,i,f_out);
-      }
-      write_int(f_out,-1);
-      write_flush(f_out);
+      recv_files(f_in,flist,local_name);
       if (verbose > 1)
-	fprintf(stderr,"generator wrote %d\n",write_total());
+	fprintf(stderr,"receiver read %d\n",read_total());
       exit(0);
     }
 
-    recv_files(f_in,flist,local_name);
-    report(f_in);
+    for (i = 0; i < flist->count; i++) {
+      if (S_ISDIR(flist->files[i].mode)) {
+	if (dry_run) continue;
+	if (mkdir(flist->files[i].name,flist->files[i].mode) != 0 &&
+	    errno != EEXIST) {
+	  fprintf(stderr,"mkdir %s : %s\n",
+		  flist->files[i].name,strerror(errno));
+	}
+	continue;
+      }
+      recv_generator(local_name?local_name:flist->files[i].name,
+		     flist,i,f_out);
+    }
+    write_int(f_out,-1);
+    write_flush(f_out);
     if (verbose > 1)
-      fprintf(stderr,"receiver read %d\n",read_total());
-    waitpid(pid, &status, 0);
+      fprintf(stderr,"generator wrote %d\n",write_total());
+    
     waitpid(pid2, &status2, 0);
 
+    report(f_in);
+
+    waitpid(pid, &status, 0);
+
     return status | status2;
 }
