@@ -20,6 +20,7 @@
 #include "rsync.h"
 
 extern int verbose;
+extern int am_server;
 extern int always_checksum;
 extern time_t starttime;
 
@@ -34,6 +35,7 @@ extern int preserve_devices;
 extern int preserve_uid;
 extern int preserve_gid;
 extern int preserve_times;
+extern int dry_run;
 
 /*
   free a sums struct
@@ -183,6 +185,8 @@ static struct sum_struct *receive_sums(int f)
 
 static void set_perms(char *fname,struct file_struct *file)
 {
+  if (dry_run) return;
+
 #ifdef HAVE_UTIME
   if (preserve_times) {
     struct utimbuf tbuf;  
@@ -240,13 +244,13 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 	  return;
       }
     }
-    unlink(fname);
-    if (symlink(flist->files[i].link,fname) != 0) {
+    if (!dry_run) unlink(fname);
+    if (!dry_run && symlink(flist->files[i].link,fname) != 0) {
       fprintf(stderr,"link %s -> %s : %s\n",
 	      fname,flist->files[i].link,strerror(errno));
     } else {
       if (verbose) 
-	fprintf(stderr,"%s -> %s\n",fname,flist->files[i].link);
+	fprintf(am_server?stderr:stdout,"%s -> %s\n",fname,flist->files[i].link);
     }
     return;
   }
@@ -258,15 +262,15 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
     if (statret != 0 || 
 	st.st_mode != flist->files[i].mode ||
 	st.st_rdev != flist->files[i].dev) {	
-      unlink(fname);
+      if (!dry_run) unlink(fname);
       if (verbose > 2)
 	fprintf(stderr,"mknod(%s,0%o,0x%x)\n",
 		fname,(int)flist->files[i].mode,(int)flist->files[i].dev);
-      if (mknod(fname,flist->files[i].mode,flist->files[i].dev) != 0) {
+      if (!dry_run && mknod(fname,flist->files[i].mode,flist->files[i].dev) != 0) {
 	fprintf(stderr,"mknod %s : %s\n",fname,strerror(errno));
       } else {
 	if (verbose)
-	  fprintf(stderr,"%s\n",fname);
+	  fprintf(am_server?stderr:stdout,"%s\n",fname);
 	set_perms(fname,&flist->files[i]);
       }
     }
@@ -282,7 +286,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
   if (statret == -1) {
     if (errno == ENOENT) {
       write_int(f_out,i);
-      send_sums(NULL,f_out);
+      if (!dry_run) send_sums(NULL,f_out);
     } else {
       if (verbose > 1)
 	fprintf(stderr,"recv_generator failed to open %s\n",fname);
@@ -314,6 +318,11 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
     return;
   }
 
+  if (dry_run) {
+    write_int(f_out,i);
+    return;
+  }
+
   /* open the file */  
   fd = open(fname,O_RDONLY);
 
@@ -419,6 +428,12 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
       if (local_name)
 	fname = local_name;
 
+      if (dry_run) {
+	if (!am_server && verbose)
+	  printf("%s\n",fname);
+	continue;
+      }
+
       if (verbose > 2)
 	fprintf(stderr,"recv_files(%s)\n",fname);
 
@@ -459,8 +474,8 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
       fd2 = open(fnametmp,O_WRONLY|O_CREAT,st.st_mode);
       if (fd2 == -1) return -1;
 
-      if (verbose)
-	fprintf(stderr,"%s\n",fname);
+      if (!am_server && verbose)
+	printf("%s\n",fname);
 
       /* recv file data */
       receive_data(f_in,buf,fd2);
@@ -526,6 +541,13 @@ off_t send_files(struct file_list *flist,int f_out,int f_in)
       if (verbose > 2) 
 	fprintf(stderr,"send_files(%d,%s)\n",i,fname);
 
+      if (dry_run) {	
+	if (!am_server && verbose)
+	  printf("%s\n",fname);
+	write_int(f_out,i);
+	continue;
+      }
+
       fd = open(fname,O_RDONLY);
       if (fd == -1) {
 	fprintf(stderr,"send_files failed to open %s: %s\n",
@@ -560,6 +582,9 @@ off_t send_files(struct file_list *flist,int f_out,int f_in)
 
       if (verbose > 2)
 	fprintf(stderr,"calling match_sums %s\n",fname);
+
+      if (!am_server && verbose)
+	printf("%s\n",fname);
       
       match_sums(f_out,s,buf,st.st_size);
       write_flush(f_out);
